@@ -45,7 +45,7 @@ def select_atoms_from_list(PDB_atom_name, atoms_list):
             return atom
 
 
-def get_H_bonded_to_grow(PDB_atom_name, prody_complex, PDB_atom_to_replace=None, chain="L"):
+def get_H_bonded_to_grow(PDB_atom_name, prody_complex, PDB_atom_to_replace=None, chain="L", resnum=None):
     """
     Given a heavy atom name (string) and a complex (prody molecule) it returns the hydrogen atom of the chain L
     placed at bonding distance of the input atom name. If there is more than one, a checking of contacts with the
@@ -60,7 +60,13 @@ def get_H_bonded_to_grow(PDB_atom_name, prody_complex, PDB_atom_to_replace=None,
     # Select the hydrogens bonded to the heavy atom 'PDB_atom_name'
 
     # When non specific atom is selected we search hydrogens automatically
-    selected_atom = prody_complex.select("chain {} and hydrogen within 1.70 of name {}".format(chain, PDB_atom_name))  # Replace for 1.53 :)
+    if not resnum:
+        selected_atom = prody_complex.select("chain {} and hydrogen within 1.74 of name {}".format(chain, 
+                                                                                             PDB_atom_name))
+    else:
+        selected_atom = prody_complex.select("chain {} and hydrogen within 1.74 of name {} and resnum {}".format(chain,
+                                                                                                          PDB_atom_name,
+                                                                                                          resnum))
     # If it is selected, we have to differentiate between hydrogens or heavy atoms
     if PDB_atom_to_replace:
         print("ATOM TO REPLACE: {}".format(PDB_atom_to_replace))
