@@ -21,6 +21,14 @@ def check_and_create_DataLocal(working_dir):
     check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005")))
     check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/HeteroAtoms")))
     check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/HeteroAtoms", "templates_generated")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/Protein")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/Protein", "templates_generated")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OFF")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OFF/Protein")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OFF/Protein", "templates_generated")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OFF/Parsley")))
+    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OFF/Parsley", "templates_generated")))
+ 
 
 
 def check_and_create_results_folder(results_folder, workdir):
@@ -29,8 +37,5 @@ def check_and_create_results_folder(results_folder, workdir):
 
 
 def check_and_create_pdb_clusters_folder(pdbout_folder, iteration):
-    if iteration == 0:
-        check_and_create_folder(pdbout_folder)
-        check_and_create_folder(os.path.join(pdbout_folder, "{}".format(iteration)))
-    else:
-        check_and_create_folder(os.path.join(pdbout_folder, "{}".format(iteration)))
+    check_and_create_folder(pdbout_folder)
+    check_and_create_folder(os.path.join(pdbout_folder, "{}".format(iteration)))
